'GooeyBlob by David 'OddBall' Williamson
'''''''''' '' ''''' ''''''''' ''''''''''
'Powered by PhysLite
'''''''' '' ''''''''
'The Examples are more to show the kinds of things that can be
'acheived using PhysLite than to teach specifics of the module.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.PNGLoader
?win32
Import BRL.D3D7Max2D
?

Import "PhysLite/PhysLite.bmx"

Type TBlob Extends TBody
	
	Const SEG:Int=24
	Const CLEN:Int=SEG+SEG/2
	Const ANG:Float=360.0/SEG
	
	Field mList:TMass[SEG]
	Field cList:TConstraint[CLEN]
	Field rig:TRig
	
	Field cx:Float
	Field cy:Float
	
	Field vx:Float
	Field vy:Float
	Field vmag:Float
	
	Field dir:Int
	
	Function Create:TBlob( x:Float, y:Float )
		Local b:TBlob=New TBlob
		
		For Local count:Int=0 To SEG-1
			
			b.mList[count]=TMass.Create( x+Cos(count*ANG)*24.0,y+Sin(count*ANG)*24.0,2,0,2,2 )
			
		Next
		
		For Local count:Int=SEG/2 To CLEN-1
			
			If count<CLEN-1
				b.cList[count]=TConstraint.Create( b.mList[count-(SEG/2)],b.mList[count-((SEG/2)-1)], ,1 )
			Else
				b.cList[count]=TConstraint.Create( b.mList[SEG-1],b.mList[0], ,1 )
			EndIf
			
		Next
		For Local count:Int=0 To (SEG/4)-1
			
			b.cList[(SEG/2)+count].Assign b
			b.cList[(CLEN-1)-count].Assign b
			b.cList[(SEG-1)-count].Assign b
			b.cList[SEG+count].Assign b
			
		Next
		
		For Local count:Int=0 To (SEG/2)-1
			
			b.cList[count]=TConstraint.Create( b.mList[count],b.mList[count+(SEG/2)], ,0.1 )
			b.cList[count].Assign b
			
		Next
		
		b.cx=x
		b.cy=y
		
		Return b
	End Function
	
	Method Update()
		
		Local stiff:Float
		Local rLen1:Float
		If KeyHit(key_up)
			stiff=1
			rLen1=60
		Else
			stiff=0.8
			rLen1=48
		EndIf
		If vmag>8
			stiff=1
			SetPhysicsIterations 6
		Else
			SetPhysicsIterations 2
		EndIf
		For Local count:Int=0 To (SEG/2)-1
			Blob.cList[count].SetStiffness stiff
			Blob.cList[count].SetRestLength rLen1
		Next
		Local rLen2:Float=(rLen1*Pi)/SEG
		For Local count:Int=SEG/2 To CLEN-1
			Blob.cList[count].SetRestLength rLen2
		Next
		
		dir=0
		If KeyDown(KEY_LEFT)
			For Local count:Int=0 To SEG-1
				Local m1:TMass=Blob.mList[count]
				Local m2:TMass
				If count<SEG-1
					m2=Blob.mList[count+1]
				Else
					m2=Blob.mList[0]
				EndIf
				Local a:Float=ATan2(m2.GetY()-m1.GetY(),m2.GetX()-m1.GetX())
				m1.Move -Cos(a)*0.1,-Sin(a)*0.1
			Next
			dir=-1
		ElseIf KeyDown(KEY_RIGHT)
			For Local count:Int=0 To SEG-1
				Local m1:TMass=Blob.mList[count]
				Local m2:TMass
				If count<SEG-1
					m2=Blob.mList[count+1]
				Else
					m2=Blob.mList[0]
				EndIf
				Local a:Float=ATan2(m2.GetY()-m1.GetY(),m2.GetX()-m1.GetX())
				m1.Move Cos(a)*0.1,Sin(a)*0.1
			Next
			dir=1
		EndIf
		
		Local oldx:Float=cx
		Local oldy:Float=cy
		cx=0
		cy=0
		For Local m:TMass=EachIn mList
			cx:+m.GetX();cy:+m.GetY()
		Next
		cx:/SEG
		cy:/SEG
		vx=cx-oldx
		vy=cy-oldy
		vmag=Sqr(vx*vx+vy*vy)
		If KeyHit(KEY_SPACE)
			Kill
			blob=TBlob.Create(level.spawnX,level.spawnY)
		EndIf
	End Method
	
	Method Kill()
		clear
	End Method
	
	Method Draw()
		Local poly:Float[SEG*4+8]
		Local poly2:Float[SEG*2]
		poly[0]=cx
		poly[1]=cy
		poly[2]=0.5
		poly[3]=0.5
		Local count:Int=4
		Local count2:Int=0
		For Local m:TMass=EachIn mList
			poly[count]=m.GetX()
			poly[count+1]=m.GetY()
			SphereMap(poly[count], poly[count+1], cx, cy, poly[count+2], poly[count+3])
			If count=4
				poly[4+SEG*4]=m.GetX()
				poly[4+SEG*4+1]=m.GetY()
				SphereMap(poly[4+SEG*4], poly[4+SEG*4+1], cx, cy, poly[4+SEG*4+2], poly[4+SEG*4+3])
			EndIf
			count:+4
			poly2[count2]=m.GetX()
			poly2[count2+1]=m.GetY()
			count2:+2
		Next
		SetColor 255,255,255
		drawtexturedpoly poly,sphere
		SetColor 0,0,63
		DrawPoly poly2,False
		Local frame:Int
		Select dir
		Case -1
			If vx<-2
				frame=2
			Else
				frame=4
			EndIf
		Case 0
			frame=0
		Case 1
			If vx>2
				frame=1
			Else
				frame=3
			EndIf
		End Select
		SetColor 0,0,0
'		For Local m:TMass=EachIn mList
'			Plot m.GetX()-0.5,m.GetY()-0.5
'		Next
		SetColor 255,255,255
		SetScale 0.5,0.5
		DrawImage face,cx,cy,frame
		SetScale 1,1
'		DrawOval cx-1,cy-1,2,2
	End Method
		
End Type

Type TLevel
	Field width:Int
	Field height:Int
	
	Field spawnX:Int
	Field spawnY:Int
	
	Field objList:TList=CreateList()
	
	Field gameTime:Int
	
	Method Reset()
		For Local o:TLevelObj=EachIn objList
			o.Reset
		Next
	End Method
	
	Method Update()
		gameTime=MilliSecs()
		For Local o:TLevelObj=EachIn objList
			o.Update
		Next
	End Method
	
	Method Draw()
		For Local o:TLevelObj=EachIn objList
			o.Draw
		Next
	End Method
	
End Type

Type TLevelObj Abstract
	
	Method Reset()
	End Method
	
	Method Update()
	End Method
	
	Method Draw()
	End Method
	
End Type

Type TScenery Extends TLevelObj
	Field zone:TZone
	
	Function Create:TScenery( area:Float[] )
		
		Local s:TScenery=New TScenery
		
		s.zone=TZone.Create(area,TZone.FILL,1,10)
		
		level.objList.AddLast s
		
		Return s
		
	End Function
	
	Method Draw()
		SetColor 0,127,127
		DrawPoly zone.GetArea(),True,zone.GetPositionX(),zone.GetPositionY()
		SetColor 0,0,0
		DrawPoly zone.GetArea(),False,zone.GetPositionX(),zone.GetPositionY()
	End Method
	
End Type

Type TLift Extends TLevelObj
	Field zone:TZone
	
	Field posX:Float
	Field posY:Float
	Field rate:Float
	Field range:Float
	Field style:Int
	
	Function Create:TLift( area:Float[],x:Float,y:Float,rate:Float,range:Float,style:Int=0 )
		
		Local l:TLift=New TLift
		
		l.zone=TZone.Create(area,TZone.FILL,1,20)
		l.posX=x
		l.posY=y
		l.rate=rate
		l.range=range
		l.style=style
		
		level.objList.AddLast l
		
		Return l
		
	End Function
	
	Method Update()
		
		Local y:Float
		If style
			y=Cos(level.gameTime*rate)*range+posY
		Else
			y=Sin(level.gameTime*rate)*range+posY
		EndIf
		zone.Position posX,y
		
	End Method
	
	Method Draw()
		SetColor 191,95,0
		DrawPoly zone.GetArea(),True,zone.GetPositionX(),zone.GetPositionY()
		SetColor 0,0,0
		DrawPoly zone.GetArea(),False,zone.GetPositionX(),zone.GetPositionY()
	End Method
	
End Type

Type TBridge Extends TLevelObj
	Field body:TBody
	Field mass:TMass[]
	
	Function Create:TBridge( x:Float, y:Float, length:Int )
		Local b:TBridge=New TBridge
		
		b.body=New TBody
		
		b.mass=New TMass[length]
		Local con:TConstraint[]=New TConstraint[length-1]
		
		For Local count:Int=0 To length-1
			b.mass[count]=TMass.Create(x+count*40, y, 4, 20, 1)
		Next
		
		For Local count:Int=0 To length-2
			con[count]=TConstraint.Create(b.mass[count],b.mass[count+1],,0.5)
			con[count].Assign b.body
		Next
		
		b.mass[0].Freeze
		b.mass[length-1].Freeze
		
		level.objList.AddLast b
		
		Return b
	End Function
	
	Method Draw()
		For Local m:TMass=EachIn mass
			SetColor 0,0,0
			DrawOval m.GetX()-20,m.GetY()-20,40,40
			SetColor 191,95,0
			DrawOval m.GetX()-19,m.GetY()-19,38,38
		Next
	End Method
	
End Type

Type TPlatform Extends TLevelObj
	Field body:TBody
	Field mass:TMass[]
	
	Field posX:Float
	Field posY:Float
	Field rate:Float
	Field range:Float
	Field style:Int
	
	Field oldX:Float
	
	Function Create:TPlatform( x:Float, y:Float, length:Int, rate:Float, range:Float, style:Int=0 )
		Local p:TPlatform=New TPlatform
		
		p.body=New TBody
		
		p.mass=New TMass[length]
		Local con:TConstraint[]=New TConstraint[length-1]
		
		For Local count:Int=0 To length-1
			p.mass[count]=TMass.Create(x+count*20, y, 4, 20, 1, 30)
			p.mass[count].Freeze
		Next
		
		For Local count:Int=0 To length-2
			con[count]=TConstraint.Create(p.mass[count],p.mass[count+1],,0.5)
			con[count].Assign p.body
		Next
		
		p.posX=x
		p.posY=y
		p.rate=rate
		p.range=range
		p.style=style
		
		level.objList.AddLast p
		
		Return p
	End Function
	
	Method Update()
		
		Local x:Float
		If style
			x=Cos(level.gameTime*rate)*range+posX
		Else
			x=Sin(level.gameTime*rate)*range+posX
		EndIf
		For Local count:Int=0 To mass.Length-1
			mass[count].Position x+count*20,posY
		Next
	End Method
	
	Method Draw()
		SetColor 0,0,0
		For Local m:TMass=EachIn mass
			DrawOval m.GetX()-20,m.GetY()-20,40,40
		Next
		SetColor 191,95,0
		For Local m:TMass=EachIn mass
			DrawOval m.GetX()-19,m.GetY()-19,38,38
		Next
	End Method
	
End Type

Graphics 400,300

Local w:Int=1024
Local h:Int=768
Local d:Int=32
Local m:Int=0
Local s:Int=3

Repeat
	
	Cls
	
	SetColor 255,255,255
	DrawText "Select graphics mode",0,0
	DrawText "--------------------",0,15
	DrawText "[1] - 800x600 (windowed)",0,30
	DrawText "[2] - 800x600",0,45
	DrawText "[3] - 1024x768",0,60
	DrawText "[4] - 1280x768 (widescreen)",0,75
	?win32
	DrawText "[x] - DirectX",0,105
	DrawText "[g] - OpenGL",0,120
	?
	DrawText "[ENTER] - To continue",0,150
	
	SetColor 255,Sin(MilliSecs()*0.25)*127+127,0
	Select s
	Case 1
		DrawText "[1] - 800x600 (windowed)",0,30
	Case 2
		DrawText "[2] - 800x600",0,45
	Case 3
		DrawText "[3] - 1024x768",0,60
	Case 4
		DrawText "[4] - 1280x768 (widescreen)",0,75
	End Select
	?win32
	Select m
	Case 0
		DrawText "[x] - DirectX",0,105
	Case 1
		DrawText "[g] - OpenGL",0,120
	End Select
	?
	If KeyHit(KEY_1)
		w=800
		h=600
		d=0
		s=1
	EndIf
	If KeyHit(KEY_2)
		w=800
		h=600
		d=32
		s=2
	EndIf
	If KeyHit(KEY_3)
		w=1024
		h=768
		d=32
		s=3
	EndIf
	If KeyHit(KEY_4)
		w=1280
		h=768
		d=32
		s=4
	EndIf
	?win32
	If KeyHit(KEY_x) Then m=0
	If KeyHit(KEY_g) Then m=1
	?
	
	Flip
	
Until KeyHit(KEY_ENTER)

EndGraphics

Global GWIDTH:Int=w
Global GHEIGHT:Int=h
Global GXCENTER:Int=GWIDTH/2
Global GYCENTER:Int=GHEIGHT/2
Global GDEPTH:Int=d
Const GHERTZ:Int=60

?win32
If m Then SetGraphicsDriver GLMax2DDriver()
?

Graphics GWIDTH,GHEIGHT,GDEPTH,GHERTZ
If d Then HideMouse

PhysicsBegin 2048,1024,0.1, ,2

Global level:TLevel=New TLevel
level.spawnX=25;level.spawnY=850
'level.spawnX=1050;level.spawnY=650
'level.spawnX=25;level.spawnY=300
'level.spawnX=1150;level.spawnY=25
level.width=2048;level.height=1024

PhysicsSetBounds 0,0,level.width,level.height

Global Blob:TBlob=TBlob.Create( level.spawnX,level.spawnY )
Global face:TImage=LoadAnimImage("media/faces.png",64,64,0,5,FILTEREDIMAGE|MIPMAPPEDIMAGE)
SetImageHandle face,32,32
Global sphere:TImage=LoadImage("media/sphere.png")
Global silver:Int=False

TLift.Create( [0.0,0.0,100.0,0.0,100.0,450.0,0.0,450.0],1900,775,0.05,100 )
TLift.Create( [0.0,0.0,100.0,0.0,100.0,100.0,0.0,100.0],1700,700,0.15,25 )
TLift.Create( [0.0,0.0,100.0,0.0,100.0,100.0,0.0,100.0],1500,700,0.15,25,True )
TLift.Create( [0.0,0.0,100.0,0.0,100.0,100.0,0.0,100.0],1300,700,-0.15,25 )
TLift.Create( [0.0,0.0,100.0,0.0,100.0,150.0,0.0,150.0],900,650,0.15,25 )
TLift.Create( [0.0,0.0,100.0,0.0,100.0,200.0,0.0,200.0],700,600,0.15,25,True )
TLift.Create( [0.0,0.0,100.0,0.0,100.0,250.0,0.0,250.0],500,550,-0.15,25 )
TLift.Create( [0.0,0.0,100.0,0.0,100.0,250.0,0.0,250.0],0,400,-0.05,75 )

TBridge.Create( 1030,705,7 )

TPlatform.Create(1250,145,5,0.05,100)
TPlatform.Create(880,145,5,-0.05,100)

TScenery.Create( [0.0,875.0,100.0,875.0,299.0,1074.0,0.0,1074.0] )
TScenery.Create( [175.0,875.0,500.0,875.0,500.0,974.0,274.0,974.0] )
TScenery.Create( [600.0,875.0,700.0,875.0,700.0,974.0,600.0,974.0] )
TScenery.Create( [800.0,875.0,900.0,875.0,900.0,974.0,800.0,974.0] )
TScenery.Create( [1000.0,875.0,1050.0,875.0,1249.0,1074.0,1000.0,1074.0] )
TScenery.Create( [1125.0,875.0,1300.0,875.0,1300.0,974.0,1224.0,974.0] )
TScenery.Create( [1400.0,875.0,1500.0,875.0,1500.0,974.0,1400.0,974.0] )
TScenery.Create( [1600.0,875.0,1700.0,875.0,1700.0,974.0,1600.0,974.0] )
TScenery.Create( [1800.0,875.0,1900.0,875.0,1900.0,1024.0,1800.0,1024.0] )
TScenery.Create( [2000.0,450.0,500.0,450.0,500.0,375.0,1550.0,375.0,1550.0,125.0,1750.0,125.0,1900.0,275.0,2048.0,275.0,2048.0,1024.0,2000.0,1024.0] )
TScenery.Create( [0.0,775.0,1000.0,775.0,1000.0,725.0,800.0,725.0,700.0,675.0,600.0,675.0,500.0,625.0,400.0,625.0,100.0,475.0,0.0,475.0] )
TScenery.Create( [175.0,325.0,575.0,125.0,710.0,125.0,710.0,325.0,400.0,325.0,400.0,550.0,175.0,437.5] )


SetClsColor 127,127,127
SetLineWidth 1
SetBlend ALPHABLEND

Repeat
	
	Local time:Int=MilliSecs()
	Blob.Update
	
	level.Update
	
	PhysicsUpdate
	
	If blob.cx>1900 And blob.cy<275 Then silver=True
	If silver=True And blob.mList[0].GetMass()<4
		sphere=LoadImage("media/sphere4.png")
		For Local m:TMass=EachIn blob.mList
			m.SetMass 4
			m.SetFriction 0
		Next
	EndIf
	
	Local originX:Float=GXCENTER-blob.cx
	If originX>0 Then originX=0
	If originX<GWIDTH-level.width Then originX=GWIDTH-level.width
	Local originY:Float=GYCENTER-blob.cy
	If originY>0 Then originY=0
	If originY<GHEIGHT-level.height Then originY=GHEIGHT-level.height
	SetOrigin originX,originY
	
	Cls
	
	SetColor 127,0,0
	DrawText " START ",0,800
	DrawText "KEEP GOING",1125,800
	DrawText "MIND YOUR STEP",1495,595
	DrawText "  UP  ",420,545
	DrawText "ALMOST THERE",710,45
	DrawText " CONGRATULATIONS ",1900,195
	SetColor 0,0,0
	DrawText "> > > >",0,815
	DrawText "> > > > > ",1125,815
	DrawText "< < < < < < < ",1495,610
	DrawText "< < < ",420,560
	DrawText "> > > > > > ",710,60
	DrawText "* * * * * * * * *",1900,180
	DrawText "* * * * * * * * *",1900,210
	SetColor 255,255,0
	DrawText " > > > ",0,815
	DrawText " > > > > >",1125,815
	DrawText " < < < < < < <",1495,610
	DrawText " < < <",420,560
	DrawText " > > > > > >",710,60
	DrawText " * * * * * * * * ",1900,180
	DrawText "*               *",1900,195
	DrawText " * * * * * * * * ",1900,210
	
	Blob.Draw
	
	level.Draw
	
	SetRotation 0

	SetOrigin 0,0
	SetColor 0,0,0
	DrawText "Gooey Blob (Prototype) by Dave 'Oddball' Williamson",17,17
	DrawText "----- ---- ----------- -- ---- --------- ----------",17,32
	DrawText "[LEFT ARROW] - Roll left",17,62
	DrawText "[RIGHT ARROW] - Roll right",17,77
	DrawText "[UP ARROW] - Bounce/Jump",17,92
	DrawText "[SPACEBAR] - Reset blob",17,122
	DrawText "[ESCAPE] - Exit",17,152
	DrawText "Powered by PhysLite",GWIDTH-173,GHEIGHT-28
	SetColor 255,255,255
	SetColor 0,0,255
	DrawText "Gooey Blob (Prototype) by Dave 'Oddball' Williamson",15,15
	SetColor 0,32,255
	DrawText "----- ---- ----------- -- ---- --------- ----------",15,30
	SetColor 0,63,255
	DrawText "[LEFT ARROW] - Roll left",15,60
	SetColor 0,95,255
	DrawText "[RIGHT ARROW] - Roll right",15,75
	SetColor 0,127,255
	DrawText "[UP ARROW] - Bounce/Jump",15,90
	SetColor 0,159,255
	DrawText "[SPACEBAR] - Reset blob",15,120
	SetColor 0,191,255
	DrawText "[ESCAPE] - Exit",15,150
	SetColor 255,255,255
	DrawText "Powered by",GWIDTH-175,GHEIGHT-30
	SetColor 255,0,0
	DrawText "Phys",GWIDTH-175+88,GHEIGHT-30
	SetColor 0,0,255
	DrawText "    Lite",GWIDTH-175+88,GHEIGHT-30

	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End

'SphereMap code by IndiePath

' works in world space and object space BUT don't mix!
' pass co-ordinates and the texture co-ordinates will be returned
' x#,y# = vertex co-ordinate
' midx#, midy# - center of object - maybe attach this to a spring!
' u#, v# - texture co-ordinates returned by function

Function SphereMap(x#,y#, midx#=0, midy#=0,u# Var,v# Var)
	Local xd# = x - midx
	Local yd# = y - midy 
	Local Mag:Double = Sqr(xd*xd + yd*yd)
	xd = xd / mag
	yd = yd / mag
	u =  xd*0.5 + 0.5
	v =  yd*0.5 + 0.5
End Function

'DrawTexturedPoly code by Yan
Function DrawTexturedPoly(xyuv:Float[], image:TImage, frame:Int=0, fan:Int=1)
	Local textureWidthRatio! = Double(image.width) / Pow2Size(image.width)
	Local textureHeightRatio! = Double(image.height) / Pow2Size(image.height)
	Local hx#, hy#, ox#, oy#
	
	GetHandle(hx#, hy#)
	GetOrigin(ox#, oy#)
?Win32
	Local DXDriver:TD3D7Max2DDriver = TD3D7Max2DDriver(_max2dDriver)	
	If DXDriver
		'DX
		Local verts#[(xyuv#.length / 4) * 6]
		Local vCount:Int
		
		For Local c:Int=0 Until xyuv#.length Step 4
			verts#[vCount] = ((xyuv#[c] - hx#) * DXDriver.ix#) + ((xyuv#[c + 1] - hy#) * DXDriver.iy#) + ox#
			verts#[vCount + 1] = ((xyuv#[c] - hx#) * DXDriver.jx#) + ((xyuv#[c + 1] - hy#) * DXDriver.jy#) + oy#
			verts#[vCount + 2] = 0
			(Int Ptr(Float Ptr(verts#)))[vCount + 3] = DXDriver.drawcolor
			verts#[vCount + 4] = xyuv#[c + 2] * textureWidthRatio!
			verts#[vCount + 5] = xyuv#[c + 3] * textureHeightRatio!
			
			vCount :+ 6
		Next

			DXDriver.SetActiveFrame(TD3D7ImageFrame(image.Frame(frame)))
			If fan
				DXDriver.device.DrawPrimitive(D3DPT_TRIANGLEFAN, D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1, verts#, xyuv#.length / 4, 0)
			Else
				DXDriver.device.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1, verts#, xyuv#.length / 4, 0)
			EndIf
	Else
?
		'GL
		'Can't get at GLMax2D's transform info, so...
		Local sx#, sy#
		GetScale(sx#, sy#)
		
		glPushMatrix()
	
		glBindTexture(GL_TEXTURE_2D, TGLImageFrame(image.Frame(frame)).name)
		glEnable(GL_TEXTURE_2D)	
		
		glTranslatef(ox#, oy#, 0.0)		
		glRotatef(GetRotation(), 0.0, 0.0, 1.0)
		glScalef(sx#, sy#, 1.0) 
		
		If fan Then glBegin(GL_POLYGON) Else glBegin(GL_TRIANGLE_STRIP)
		
		For Local c:Int=0 Until xyuv#.length Step 4
			glTexCoord2f(xyuv#[c + 2] * textureWidthRatio!, xyuv#[c + 3] * textureHeightRatio!)
			glVertex2f(xyuv#[c] - hx#, xyuv#[c + 1] - hy#)
		Next
		
		glEnd
	
		glDisable(GL_TEXTURE_2D)
		glPopMatrix()
?Win32
	EndIf
?
End Function

Function Pow2Size:Int(n:Int)
	Local t:Int = 1
	
	While t < n
		t :* 2
	Wend
	
	Return t
End Function

'Alternative DrawPoly function
Function DrawPoly( xy:Float[], fill:Int=True, x:Float=0, y:Float=0 )
	Local origin_x:Float
	Local origin_y:Float
	GetOrigin origin_x,origin_y
	Local handle_x:Float
	Local handle_y:Float
	GetHandle handle_x,handle_y
	
	If fill
		_max2dDriver.DrawPoly xy,..
		handle_x,handle_y,..
		x+origin_x,y+origin_y
	Else
		Local x1:Float=xy[xy.Length-2]
		Local y1:Float=xy[xy.Length-1]
		For Local i:Int=0 Until Len xy Step 2
			Local x2:Float=xy[i]
			Local y2:Float=xy[i+1]
			_max2dDriver.DrawLine..
			handle_x+x1,handle_y+y1,..
			handle_x+x2,handle_y+y2,..
			x+origin_x-0.5,y+origin_y-0.5
			x1=x2
			y1=y2
		Next
	EndIf
End Function