'Tutorial01: A box
''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'18th May 2007
''''''''''''''
'Description
''''''''''''
'How to create a basic physics simulation using PhysLite.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D

'Import PhysLite so we can use it
Import "PhysLite/PhysLite.bmx"

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

'Now lets start the simulation with the PhysicsBegin command
'The first two values (800,600) are the size of the simulation
'The third (0.1) is gravity
PhysicsBegin 800,600,0.1

'Now lets make our box
Global mass:TMass[4]

'We use TMass.Create to make a mass
'The first two values (390,290) are the initial position
'The third (10) is the actual mass of the TMass
'And finally there's friction (1)
'We've left the final parameter (colGroup) blank
'This will make PhysLite select a unique colGroup for the mass
mass[0]=TMass.Create(390,290,10,20,1)
'Now lets make the rest
mass[1]=TMass.Create(410,290,10,20,1,mass[0].GetColGroup())
mass[2]=TMass.Create(390,310,10,20,1,mass[0].GetColGroup())
mass[3]=TMass.Create(410,310,10,20,1,mass[0].GetColGroup())
'Notice we are setting the final parameter (colGroup)
'We do this so that all the Tmass objects that make
'up our box belong to the same collision group

'Next it's time to join them all up
Global con:TConstraint[6]

'For this we use TConstraints
'To make a constraint we use TConstraint.Create
'Create takes four parameters but for now well just use the first two
'These first two parameters are the TMass objects that the constraint connects
con[0]=TConstraint.Create(mass[0],mass[1])
con[1]=TConstraint.Create(mass[0],mass[2])
con[2]=TConstraint.Create(mass[0],mass[3])
con[3]=TConstraint.Create(mass[1],mass[2])
con[4]=TConstraint.Create(mass[1],mass[3])
con[5]=TConstraint.Create(mass[2],mass[3])

'And finally for the constraints to become active they must be assigned to a TBody
Global body:TBody=New TBody
For Local count:Int=0 To 5
	'The Assign method takes a TBody as it's only parameter
	con[count].Assign body
Next
'And that's it our box is complete

'So lets see what it's like in motion
Repeat
	
	'The PhysicsUpdate function steps the simulation through to the next time step
	PhysicsUpdate
	
	Cls
	SetColor 0,0,255
	For Local m:TMass=EachIn mass
		'The Draw method draws the mass at it's current location. It is intended more for
		'debugging than for proper use.
		m.Draw
	Next
	SetColor 255,255,255
	For Local c:TConstraint=EachIn con
		'Constraints can be drawn in the same way as a TMass.
		c.Draw
	Next
	Flip
	
Until KeyHit(KEY_ESCAPE)

'Make sure we tidy up before leaving
PhysicsEnd

EndGraphics

End
'The end
