'Tutorial04: It's freezing
''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'18th May 2007
''''''''''''''
'Description
''''''''''''
'Using the Freeze method to anchor a mass to a point.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import "PhysLite/PhysLite.bmx"

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

PhysicsBegin 800,600,0.1,,2

Global mlist:TList=New TList
Global clist:TList=New TList
Global blist:TList=New TList

Type TBox Extends TBody
	
	Function Create:TBox( x:Float,y:Float )
		Local box:TBox=New TBox
		
		Local mass:TMass[4]
		mass[0]=TMass.Create(x-10,y-10,10,20,1)
		mass[1]=TMass.Create(x+10,y-10,10,20,1,mass[0].GetColGroup())
		mass[2]=TMass.Create(x-10,y+10,10,20,1,mass[0].GetColGroup())
		mass[3]=TMass.Create(x+10,y+10,10,20,1,mass[0].GetColGroup())
		For Local count:Int=0 To 3
			mlist.AddLast mass[count]
		Next
		
		Local con:TConstraint[6]
		con[0]=TConstraint.Create(mass[0],mass[1])
		con[1]=TConstraint.Create(mass[0],mass[2])
		con[2]=TConstraint.Create(mass[0],mass[3])
		con[3]=TConstraint.Create(mass[1],mass[2])
		con[4]=TConstraint.Create(mass[1],mass[3])
		con[5]=TConstraint.Create(mass[2],mass[3])
		For Local count:Int=0 To 5
			con[count].Assign box
			clist.AddLast con[count]
		Next
		
		Return box
	End Function
End Type

For Local y:Int=100 To 340 Step 80
	For Local x:Int=80 To 720 Step 80
		blist.AddLast TBox.Create(x,y)
	Next
Next

'To demonstrate the Freeze method lets create a rope bridge.
'We could use a TZone to make bridge, but as this will be a
'rope bridge it needs to swing in the middle but be anchored
'at each end and that can't be done with a TZone.
Global bridge:TBody=New TBody
'First lets create our start and end point.
Global start:TMass=TMass.Create(240,400,10,20,1,99)
Global finish:TMass=TMass.Create(560,400,10,20,1,99)
'Notice that this time I've explicitly set the collision
'group (99). Don't worry about it conflicting with auto generated
'collision groups. Auto gen. only creates negative collision groups.
'As long as we only name positive value we'll be fine.

Local m1:TMass=start
Local m2:TMass
Local c:TConstraint
For Local x:Int=240 To 560 Step 40
	'Wecreate a string of TMass objects and link them all up.
	m2=TMass.Create(x,440,10,20,1,99)
	c=TConstraint.Create(m1,m2)
	c.Assign bridge
	mlist.AddLast m2
	clist.AddLast c
	m1=m2
Next
c=TConstraint.Create(m1,finish)
c.Assign bridge
clist.AddLast c

'Finally here's where the magic happens. We Freeze the start and
'finish points. This tells the sinulation not to move these TMass
'objects.
start.Freeze
finish.Freeze

'Lets see how it looks.

Repeat
	
	PhysicsUpdate
	
	Cls
	SetColor 0,0,255
	For Local m:TMass=EachIn mlist
		m.Draw
	Next
	SetColor 255,0,0
	start.Draw
	finish.Draw
	SetColor 255,255,255
	For Local c:TConstraint=EachIn clist
		c.Draw
	Next
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end
