'Tutorial10: When Worlds Collide.
'''''''''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'25th June 2007
'''''''''''''''
'Description
''''''''''''
'Collision groups are very important. Sometimes you don't want two objects
'to affect each other and this tutorial shows how that's possible.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.PNGLoader
Import "PhysLite/PhysLite.bmx"

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

PhysicsBegin 800,600,0.1,,1

Global image:TImage=LoadImage("media/bmax.png",FILTEREDIMAGE)
SetImageHandle image,20,20

Type TBox Extends TBody
	Field rig:TRig
	Field cg:Int
	
	'The create function now takes a new parameter. This will be
	'the box's collision group. Previously we've allowed PhysLite
	'to choose the colGroup by using -1.
	Function Create:TBox( x:Float,y:Float,cg:Int )
		Local box:TBox=New TBox
		
		Local mass:TMass[4]
		mass[0]=TMass.Create(x-10,y-10,10,20,1,cg)
		mass[1]=TMass.Create(x+10,y-10,10,20,1,cg)
		mass[2]=TMass.Create(x-10,y+10,10,20,1,cg)
		mass[3]=TMass.Create(x+10,y+10,10,20,1,cg)
		
		Local con:TConstraint[6]
		con[0]=TConstraint.Create(mass[0],mass[1])
		con[1]=TConstraint.Create(mass[2],mass[0])
		con[2]=TConstraint.Create(mass[0],mass[3])
		con[3]=TConstraint.Create(mass[1],mass[2])
		con[4]=TConstraint.Create(mass[3],mass[1])
		con[5]=TConstraint.Create(mass[2],mass[3])
		For Local count:Int=0 To 5
			con[count].Assign box
		Next
		
		box.rig=TRig.Create(image,mass[0],con[0])
		
		box.cg=cg
		
		Return box
	End Function
	
End Type

Global boxes:TList=New TList
Global zlist:TList=New TList
'Now we creat three boxes and three collision zones with the colGroups 0, 1, and 2.
'The box with colGroup 0(Leftmost) will not collide with any box or TZone. Likewise the
'TZone with colGroup 0(Topmost) will not collide with any of the boxes. The other boxes
'and TZones will only collide with objects that do NOT have the same colGroup as themselves.
'Put simple collision group 0 collides with nothing, and collision group values that are the
'same do NOT collide with each other.
For Local count:Int=0 To 2
	boxes.AddLast TBox.Create(250+150*count,30,count)
	zlist.AddLast TZone.Create([50.0,125.0+150*count,750.0,125.0+150*count,750.0,175.0+150*count,50.0,175.0+150*count],TZone.FILL,,count)
Next

'Let's see what happens when it's set in motion

SetClsColor 127,127,127

Repeat
	
	PhysicsUpdate
	
	Cls
	DrawTutorial
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end

Function DrawTutorial()
	For Local box:TBox=EachIn boxes
		Local cg:Int=box.cg
		SetColor (cg=0)*255,(cg=1)*255,(cg=2)*255
		box.rig.Draw
	Next
	
	SetRotation 0
	For Local zone:TZone=EachIn zlist
		Local cg:Int=zone.GetColGroup()
		SetColor (cg=0)*255,(cg=1)*255,(cg=2)*255
		zone.Draw
	Next
End Function
