'Tutorial12: Advanced Body Building.
''''''''''''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'26th June 2007
'''''''''''''''
'Description
''''''''''''
'This tutorial serves as an introduction to the TAdvBody type. So here's
'Tutorial11.bmx again, but this time using TAdvBody. Load up Tutorial11.bmx
'to compare the methods used.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.Random
Import "PhysLite/PhysLite.bmx"

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

PhysicsBegin 800,600,0.1,,1

'First of all we declare our body as a TAdvBody.
Global body:TAdvBody=New TAdvBody

'Remember in tutorial 11 that we used a list to store all the TMass types.
'Well with the TAdvBody that's not needed. The TAdvBody remembers which TMass
'types are associated with it.
For Local count:Int=0 To 5
	'MassAdd creates a mass and adds it to our body. MassAdd also returns the
	'index of the created TMass, but we don't need that for this tutorial.
	body.MassAdd(Rnd(300,500),Rnd(200,400),10,10,1,1)
Next
'Creating the constraints is even easier.
body.ConAddAll 0.5
'ConAddAll creates constraints that connect all the masses together. It doesn't
'make the most efficient TAdvBody, but for TAdvBody's with few masses it's more
'than adiquate.

'Once a TAdvBody is created, or loaded or copied, it needs to be activated before
'it becomes live in the simulation.
body.Activate

Repeat
	
	Local mx:Int=MouseX()
	Local my:Int=MouseY()
	Local button:Int=0
	If MouseDown(MOUSE_LEFT) Then button=1
	If MouseDown(MOUSE_RIGHT) Then button=2
	
	Local nearDist:Float=400
	Local nearMass:Int=-1
	
	If button=1
		'All the masses in a TAdvBody are referenced by index so first we need to
		'know the range of indices within the TAdvBody. The lowest possible index is
		'always 0, but remember the actual lowest may be higher than this. We find the
		'highest possible index by using MasMaxIndex.
		For Local index:Int=0 To body.MassMaxIndex()
			'Not every index within the range will have a mass associated with it so we
			'need to check there is a mass there before we carry on.
			If body.MassExists(index)
				'The TAdvBody methods are almost identical to the component methods except
				'they are prefixed with the components type, i.e. mass.GetX() becomes
				'body.MassGetX(index).
				Local dist:Float=(body.MassGetX(index)-mx)*(body.MassGetX(index)-mx)..
									+(body.MassGetY(index)-my)*(body.MassGetY(index)-my)
				If dist<nearDist
					nearDist=dist
					nearMass=index
				EndIf
			EndIf
		Next
		'When dealing with the individual components of the TAdvBody, i.e. Masses,
		'constraints, rigs and images, the first parameter is always the index of the
		'component to be affected.
		If nearMass>-1 Then body.MassPosition nearMass,mx,my
		
	ElseIf button=2
		For Local index:Int=0 To body.MassMaxIndex()
			Local dist:Float=(body.MassGetX(index)-mx)*(body.MassGetX(index)-mx)..
								+(body.MassGetY(index)-my)*(body.MassGetY(index)-my)
			If dist<nearDist
				nearDist=dist
				nearMass=index
			EndIf
		Next
		If nearMass>-1
			body.MassPosition nearMass,mx,my
			'Constraints are handled exactly the same as masses
			For Local index:Int=0 To body.ConMaxIndex()
				If body.ConExists(index)
					'Component methods that return a component type will return the index
					'of that component. So ConGetMass1 returns mass1's index and not the
					'actual TMass.
					If body.ConGetMass1(index)=nearMass Or body.ConGetMass2(index)=nearMass
						body.ConSetRestLength index
					EndIf
				EndIf
			Next
		EndIf
	EndIf
	
	If button<>2 Or nearMass=-1 Then PhysicsUpdate
	
	SetClsColor 127,127,127
	Cls
	
	SetColor 0,0,255
	For Local index:Int=0 To body.MassMaxIndex()
		If body.MassExists(index) Then body.MassDraw index
	Next
	SetColor 255,0,0
	If nearMass>-1 Then DrawOval body.MassGetX(nearMass)-10,body.MassGetY(nearMass)-10,20,20
	SetColor 255,255,255
	For Local index:Int=0 To body.ConMaxIndex()
		Local m1:Int=body.ConGetMass1(index)
		Local m2:Int=body.ConGetMass2(index)
		If button=2 And (m1=nearMass Or m2=nearMass)
			SetColor 0,255,0
		Else
			SetColor 255,255,255
		EndIf
		body.ConDraw index
	Next
	
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end
