'Tutorial15: Shifty tactics.
''''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'1st July 2007
''''''''''''''
'Description
''''''''''''
'If your computer's anything like mine(i.e. crap), then Tutorial14.bmx
'will have really made it chug. But there is hope. By using PhysLite's
'shifted grid option we can soften the workload and really start throwing
'huge amounts of objects around.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.Random
Import "PhysLite/PhysLite.bmx"

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

'We're going to have a lot of objects in this simulation so lets up the width and height.
PhysicsBegin 1600,1600,0.1,,2

'This variable will allow us to toggle the shifted grid on and off.
Global gridOn:Int=1

'The shifted grid is off by default so we need to activate it.
PhysicsUseSGrid gridOn
'Setting the size of the grid is very important. The value we provide is how many
'cells will be in each row/column. Setting this value too high can make the simulation
'unstable. See for yourself if you set the value to the maximum(64) then notice how 
'the bodies sometimes overlap and also will jump and bounce.
'To calculate the maximum possible grid size take the smallest dimension of the simulation.
'Our simulation is square so we'll use the width(1600). Then we divide this by the diameter
'of the largest TMass in the simulation(40). So 1600/40=40.
PhysicsSetGridSize 40
'The shifted grid can be turned on and off or resized at anytime. It is important to
'note that there is a certain amount of overhead when using the shifted grid. So using
'it for small simulations or when the are only a few objects will be slower than not
'using it.


'These values will keep track of the number of objects and when new ones are added.
Global numObjects:Int=0
Global releaseCount:Int=45

Global template:TAdvBody=TAdvBody.Load("media/box.plb")
Global boxes:TList=New TList

'Lets add a little scenery
Global zone:TZone=TZone.Create([150.0,950.0, 1450.0,950.0, 1450.0,1050.0, 150.0,1050.0],TZone.FILL)

'Our simulation area is much bigger than the screen so we'll use PhysicsSetDrawZoom to resize
'the drawing functions. A value of .5 means that the simulation will be drawn at half scale. And
'we use PhysicsSetDrawCoords to reposition the drawing functions. This means that position 800,1000
'in the simulation will be in the center of the screen. as this is not the center of the simulation
'some of the simulation area will be off the top of the screen.
PhysicsSetDrawCoords 800,1000
PhysicsSetDrawZoom .5

Repeat
	
	'Every 45 updates we create a new box
	releaseCount:+1
	If releaseCount>45
		Local box:TAdvBody=template.Copy()
		box.Position Rnd(200,1400),350
		box.Spin Rnd(-45,45)
		box.Activate
		boxes.AddLast box
		numObjects:+1
		releaseCount=0
	EndIf
	
	'If [g] is pressed toggle shifted grid on/off. Use this to see instantly the
	'speed gain of the shifted grid.
	If KeyHit(KEY_G)
		gridOn=1-gridOn
		PhysicsUseSGrid gridOn
	EndIf
	
	'Lets time the update so we can compare the two methods.
	Local updateTime:Int=MilliSecs()
	PhysicsUpdate
	updateTime=MilliSecs()-updateTime
	
	SetClsColor 127,127,127
	Cls
	
	SetColor 63,63,63
	'SetOrigin still affects the drawing commands and I'm using it here to draw
	'a drop shadow.
	SetOrigin 2,2
	For Local box:TAdvBody=EachIn boxes
		box.Draw
	Next
	zone.Draw
	SetColor 255,255,255
	SetOrigin 0,0
	For Local box:TAdvBody=EachIn boxes
		box.Draw
	Next
	SetColor 255,0,0
	zone.Draw
	
	SetOrigin 0,0
	SetColor 0,0,0
	DrawText "Number of obects: "+numObjects,7,7
	If gridOn Then DrawText "Shifted [G]rid: On",7,39 Else DrawText "Shifted [G]rid: Off",7,39
	DrawText "Update time: "+updateTime+"ms",7,23
	DrawText "Powered by PhysLite",627,572
	SetColor 255,255,255
	DrawText "Number of obects: "+numObjects,5,5
	If gridOn Then DrawText "Shifted [G]rid: On",5,37 Else DrawText "Shifted [G]rid: Off",5,37
	DrawText "Powered by ",625,570
	SetColor 255,0,0
	DrawText "           Phys",625,570
	SetColor 0,0,255
	DrawText "               Lite",625,570
	Local red:Int=0,green:Int=255
	If updateTime>=12 Then red=255
	If updateTime>=16 Then green=0
	SetColor red,green,0
	DrawText "Update time: "+updateTime+"ms",5,21
	
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end
