'Tutorial16: When worlds don't collide.
'''''''''''''''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'6th July 2007
''''''''''''''
'Description
''''''''''''
'This tutorial showcases PhysLite's first user requested feature, custom
'collision responses. This is achieved in the form of a callback function
'that overrides the default response. This feature is one of those that
'if you can't think of a reason to use it then you probably don't need to
'learn it.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.Random
Import "PhysLite/PhysLite.bmx"

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

PhysicsBegin 1600,1600,0.1,,2

PhysicsUseSGrid True
PhysicsSetGridSize 40

Global releaseCount:Int=45
Global template:TAdvBody=TAdvBody.Load("media/box.plb")
Global boxes:TList=New TList
Global zone:TZone=TZone.Create([750.0,775.0, 1400.0,775.0, 1400.0,875.0, 850.0,875.0, 850.0,1600.0, 750.0,1600.0],TZone.FILL)


'To show this feature in action we are going to use it to make some VERY basic water.

'First we create a collision zone to represent our water, and position it at the bottom of our
'simulation.
Global water:TZone=TZone.Create([0.0,0.0, 800.0,0.0, 800.0,800.0, 0.0,800.0],TZone.FILL)
water.Position 0,1200

'Next we create our callback function. This is the function that PhysLite will use instead of it's
'own default function. The parameters are the TZone and TMass involved, the collision noramal, and
'the magnitude of the penetration.
Function WaterCol(zone:TZone,mass:TMass,penNormX:Float,penNormY:Float,penMag:Float)
	'We use standard PhysLite functions to simulate the effect of water upon the mass
	Local x:Float=mass.GetVX()
	Local y:Float=mass.GetVY()
	mass.SetVelocity x*.99,y*.95
	mass.Move 0,-0.105
End Function

'Now we tell PhysLite to use our custom function instead of it's own default function when a mass
'collides with our water collision zone.
water.OverrideCollision( WaterCol )

PhysicsSetDrawCoords 800,1000
PhysicsSetDrawZoom .5
SetBlend ALPHABLEND

Repeat
	
	releaseCount:+1
	If releaseCount>45
		Local box:TAdvBody=template.Copy()
		box.Position Rnd(200,1400),350
		box.Spin Rnd(-45,45)
		box.Activate
		boxes.AddLast box
		releaseCount=0
	EndIf
	
	If water.GetPositionY()>800 Then water.Move 0,-0.1
	
	PhysicsUpdate
	
	SetClsColor 127,127,127
	Cls
	
	SetColor 63,63,63
	SetOrigin 2,2
	For Local box:TAdvBody=EachIn boxes
		box.Draw
	Next
	zone.Draw
	SetColor 255,255,255
	SetOrigin 0,0
	For Local box:TAdvBody=EachIn boxes
		box.Draw
	Next
	SetColor 127,127,255
	SetAlpha .5
	water.Draw
	SetAlpha 1
	SetColor 255,0,0
	zone.Draw
	
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end
